@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -14,50 +13,11 @@ In here we put all things (methods) that are common to workflows AND
 depend on AiiDA classes, therefore can only be used if the dbenv is loaded.
 Util that does not depend on AiiDA classes should go somewhere else.
 """
-
-from __future__ import absolute_import
-from __future__ import print_function
-import six
+import warnings
 
 from aiida.orm import Node, load_node, Bool
 from aiida.plugins import DataFactory, CalculationFactory
 
-# def is_code(code):
-#     """
-#     Test if the given input is a Code node, by object, id, uuid, or pk
-#     if yes returns a Code node in all cases
-#     if no returns None
-#     """
-#     from aiida.orm import Code
-#     from aiida.common.exceptions import NotExistent, MultipleObjectsError, InputValidationError
-
-#     if isinstance(code, Code):
-#         return code
-
-#     try:
-#         pk = int(code)
-#     except ValueError:
-#         codestring = str(code)
-#         try:
-#             code = Code.get_from_string(codestring)
-#         except NotExistent:
-#             try:
-#                 code = load_node(codestring)
-#             except NotExistent:
-#                 code = None
-#         except (InputValidationError, MultipleObjectsError):
-#             code = None
-#     else:
-#         try:
-#             code = load_node(pk)
-#         except NotExistent:
-#             code = None
-
-#     if isinstance(code, Code):
-#         return code
-#     else:
-#         return None
-
 
 def get_inputs_fleur(code, remote, fleurinp, options, label='', description='', settings=None, add_comp_para=None):
     '''
@@ -71,7 +31,8 @@ def get_inputs_fleur(code, remote, fleurinp, options, label='', description='',
     :param label: a string setting a label of the CalcJob in the DB
     :param description: a string setting a description of the CalcJob in the DB
     :param settings: additional settings of Dict type
-    :param serial: True if run a calculation in a serial mode
+    :param add_comp_para: dict with extra keys controlling the behaviour of the parallelization
+                          of the FleurBaseWorkChain
 
     Example of use::
 
@@ -84,13 +45,10 @@ def get_inputs_fleur(code, remote, fleurinp, options, label='', description='',
     Dict = DataFactory('dict')
     inputs = {}
 
+    add_comp_para_default = {'only_even_MPI': False, 'max_queue_nodes': 20, 'max_queue_wallclock_sec': 86400}
     if add_comp_para is None:
-        add_comp_para = {
-            'serial': False,
-            'only_even_MPI': False,
-            'max_queue_nodes': 20,
-            'max_queue_wallclock_sec': 86400
-        }
+        add_comp_para = {}
+    add_comp_para = {**add_comp_para_default, **add_comp_para}
 
     if remote:
         inputs['parent_folder'] = remote
@@ -107,8 +65,10 @@ def get_inputs_fleur(code, remote, fleurinp, options, label='', description='',
         inputs['label'] = label
     else:
         inputs['label'] = ''
-    # TODO check  if code is parallel version?
-    if add_comp_para['serial']:
+
+    if add_comp_para.get('serial', False):
+        warnings.warn('The serial input in add_comp_para is deprecated. Control the usage of'
+                      'MPI with the withmpi key in the options input')
         if not options:
             options = {}
         options['withmpi'] = False  # for now
@@ -116,14 +76,18 @@ def get_inputs_fleur(code, remote, fleurinp, options, label='', description='',
         #  lsf takes number of total_mpi_procs,slurm and psb take num_machines,\
         # also a full will run here mpi on that node... also not what we want.ß
         options['resources'] = {'num_machines': 1, 'num_mpiprocs_per_machine': 1}
-    else:
-        options['withmpi'] = True
 
+    if options:
+        options['withmpi'] = options.get('withmpi', True)
+        if not options['withmpi'] and 'resources' not in options:
+            # TODO not every machine/scheduler type takes number of machines
+            #  lsf takes number of total_mpi_procs,slurm and psb take num_machines,\
+            # also a full will run here mpi on that node... also not what we want.ß
+            options['resources'] = {'num_machines': 1, 'num_mpiprocs_per_machine': 1}
+
+    inputs['clean_workdir'] = Bool(add_comp_para.pop('clean_workdir', False))
     inputs['add_comp_para'] = Dict(dict=add_comp_para)
 
-    custom_commands = options.get('custom_scheduler_commands', '')
-    options['custom_scheduler_commands'] = custom_commands
-
     if settings:
         if isinstance(settings, Dict):
             inputs['settings'] = settings
@@ -220,12 +184,12 @@ def test_and_get_codenode(codenode, expected_code_type, use_exceptions=False):
         qb = QueryBuilder()
         qb.append(Code, filters={'attributes.input_plugin': {'==': expected_code_type}}, project='*')
 
-        valid_code_labels = ['{}@{}'.format(c.label, c.computer.label) for [c] in qb.all()]
+        valid_code_labels = [f'{c.label}@{c.computer.label}' for [c] in qb.all()]
 
         if valid_code_labels:
             msg = ('Given Code node is not of expected code type.\n'
                    'Valid labels for a {} executable are:\n'.format(expected_code_type))
-            msg += '\n'.join('* {}'.format(l) for l in valid_code_labels)
+            msg += '\n'.join(f'* {l}' for l in valid_code_labels)
 
             if use_exceptions:
                 raise ValueError(msg) from exc
@@ -299,7 +263,7 @@ def determine_favorable_reaction(reaction_list, workchain_dict):
     # then sort the given list from (lowest if negativ energies to highest)
     energy_sorted_reactions = []
     formenergy_dict = {}
-    for compound, uuid in six.iteritems(workchain_dict):
+    for compound, uuid in workchain_dict.items():
         # TODO ggf get formation energy from output node, or extras
         if isinstance(uuid, float):  # allow to give values
             formenergy_dict[compound] = uuid
@@ -322,7 +286,7 @@ def determine_favorable_reaction(reaction_list, workchain_dict):
                         except (AttributeError, KeyError, ValueError):  # TODO: Check this
                             ouputnode = None
                             formenergy = None
-                            print(('WARNING: output node of {} not found. I skip'.format(n)))
+                            print(f'WARNING: output node of {n} not found. I skip')
                             continue
                     formenergy = ouputnode.get('formation_energy')
                     # TODO is this value per atom?
@@ -351,30 +315,30 @@ def performance_extract_calcs(calcs):
     Note: Is not the fastest for many calculations > 1000.
     """
     data_dict = {
-        u'n_symmetries': [],
-        u'n_spin_components': [],
-        u'n_kpoints': [],
-        u'n_iterations': [],
-        u'walltime_sec': [],
-        u'walltime_sec_per_it': [],
-        u'n_iterations_total': [],
-        u'density_distance': [],
-        u'computer': [],
-        u'n_atoms': [],
-        u'kmax': [],
-        u'cost': [],
-        u'costkonstant': [],
-        u'walltime_sec_cor': [],
-        u'total_cost': [],
-        u'fermi_energy': [],
-        u'bandgap': [],
-        u'energy': [],
-        u'force_largest': [],
-        u'ncores': [],
-        u'pk': [],
-        u'uuid': [],
-        u'serial': [],
-        u'resources': []
+        'n_symmetries': [],
+        'n_spin_components': [],
+        'n_kpoints': [],
+        'n_iterations': [],
+        'walltime_sec': [],
+        'walltime_sec_per_it': [],
+        'n_iterations_total': [],
+        'density_distance': [],
+        'computer': [],
+        'n_atoms': [],
+        'kmax': [],
+        'cost': [],
+        'costkonstant': [],
+        'walltime_sec_cor': [],
+        'total_cost': [],
+        'fermi_energy': [],
+        'bandgap': [],
+        'energy': [],
+        'force_largest': [],
+        'ncores': [],
+        'pk': [],
+        'uuid': [],
+        'serial': [],
+        'resources': []
     }
     count = 0
     for calc in calcs:
@@ -388,21 +352,21 @@ def performance_extract_calcs(calcs):
         try:
             efermi = res.fermi_energy
         except AttributeError:
-            print(('skipping {}, {}'.format(pk, calc.uuid)))
+            print(f'skipping {pk}, {calc.uuid}')
             efermi = -10000
             continue  # we skip these entries
         try:
             gap = res.bandgap
         except AttributeError:
             gap = -10000
-            print(('skipping 2 {}, {}'.format(pk, calc.uuid)))
+            print(f'skipping 2 {pk}, {calc.uuid}')
             continue
 
         try:
             energy = res.energy
         except AttributeError:
             energy = 0.0
-            print(('skipping 3 {}, {}'.format(pk, calc.uuid)))
+            print(f'skipping 3 {pk}, {calc.uuid}')
             continue
 
         data_dict['bandgap'].append(gap)
@@ -420,7 +384,7 @@ def performance_extract_calcs(calcs):
         data_dict['n_iterations'].append(niter)
         data_dict['n_iterations_total'].append(res.number_of_iterations_total)
 
-        if u'overall_density_convergence' not in res_keys:
+        if 'overall_density_convergence' not in res_keys:
             data_dict['density_distance'].append(res.density_convergence)
         else:  # magnetic, old
             data_dict['density_distance'].append(res.overall_density_convergence)
@@ -598,7 +562,7 @@ def optimize_calc_options(nodes,
         raise ValueError(message)
     elif best_suggestion[1] * best_suggestion[2] == cpus_per_node:
         if best_suggestion[0] != nodes:
-            message = ('WARNING: Changed the number of nodes from {} to {}' ''.format(nodes, best_suggestion[0]))
+            message = f'WARNING: Changed the number of nodes from {nodes} to {best_suggestion[0]}'
         else:
             message = ('Computational setup is perfect! Nodes: {}, MPIs per node {}, OMP per MPI '
                        '{}. Number of k-points is {}'.format(best_suggestion[0], best_suggestion[1], best_suggestion[2],
@@ -642,3 +606,20 @@ def find_last_submitted_workchain(restart_wc):
         return calls[-1].node.uuid
     else:
         raise NotExistent
+
+
+def find_nested_process(wc_node, p_class):
+    '''
+    This function finds all nested child processes of p_class
+    '''
+    child_process = []
+    lower = wc_node.get_outgoing().all()
+    for i in lower:
+        try:
+            if i.node.process_class is p_class:
+                child_process.append(i.node)
+            else:
+                child_process.extend(find_nested_process(i.node, p_class))
+        except:  #pylint: disable=bare-except
+            pass
+    return child_process
