@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -20,7 +19,7 @@ from ase.dft.kpoints import bandpath
 import numpy as np
 
 from aiida.orm import Code, Dict, RemoteData, KpointsData
-from aiida.orm import load_node, CalcJobNode, FolderData
+from aiida.orm import load_node, CalcJobNode, FolderData, BandsData, XyData
 from aiida.engine import WorkChain, ToContext, if_
 from aiida.engine import calcfunction as cf
 from aiida.common.exceptions import NotExistent
@@ -45,7 +44,7 @@ class FleurBandDosWorkChain(WorkChain):
     # wf_parameters: {  'tria', 'nkpts', 'sigma', 'emin', 'emax'}
     # defaults : tria = True, nkpts = 800, sigma=0.005, emin= , emax =
 
-    _workflowversion = '0.4.1'
+    _workflowversion = '0.5.2'
 
     _default_options = {
         'resources': {
@@ -70,7 +69,6 @@ class FleurBandDosWorkChain(WorkChain):
         'emin': -0.50,
         'emax': 0.90,
         'add_comp_para': {
-            'serial': False,
             'only_even_MPI': False,
             'max_queue_nodes': 20,
             'max_queue_wallclock_sec': 86400
@@ -104,6 +102,8 @@ class FleurBandDosWorkChain(WorkChain):
 
         spec.output('output_banddos_wc_para', valid_type=Dict)
         spec.output('last_calc_retrieved', valid_type=FolderData)
+        spec.output('output_banddos_wc_bands', valid_type=BandsData, required=False)
+        spec.output('output_banddos_wc_dos', valid_type=XyData, required=False)
 
         spec.exit_code(230, 'ERROR_INVALID_INPUT_PARAM', message='Invalid workchain parameters.')
         spec.exit_code(231, 'ERROR_INVALID_INPUT_CONFIG', message='Invalid input configuration.')
@@ -122,12 +122,14 @@ class FleurBandDosWorkChain(WorkChain):
         '''
         ### input check ### ? or done automaticly, how optional?
         # check if fleuinp corresponds to fleur_calc
-        self.report('started bandsdos workflow version {}'.format(self._workflowversion))
+        self.report(f'started bandsdos workflow version {self._workflowversion}')
         #print("Workchain node identifiers: ")#'{}'
         #"".format(ProcessRegistry().current_calc_node))
 
         self.ctx.scf_needed = False
         self.ctx.banddos_calc = None
+        self.ctx.fleurinp_banddos = None
+        self.ctx.scf = None
         self.ctx.successful = False
         self.ctx.info = []
         self.ctx.warnings = []
@@ -145,6 +147,15 @@ class FleurBandDosWorkChain(WorkChain):
             wf_dict[key] = wf_dict.get(key, val)
         self.ctx.wf_dict = wf_dict
 
+        extra_keys = []
+        for key in self.ctx.wf_dict.keys():
+            if key not in wf_default.keys():
+                extra_keys.append(key)
+        if extra_keys:
+            error = f'ERROR: input wf_parameters for Banddos contains extra keys: {extra_keys}'
+            self.report(error)
+            return self.exit_codes.ERROR_INVALID_INPUT_PARAM
+
         defaultoptions = self._default_options
         if 'options' in inputs:
             options = inputs.options.get_dict()
@@ -161,34 +172,34 @@ class FleurBandDosWorkChain(WorkChain):
                 test_and_get_codenode(inputs.fleur, 'fleur.fleur', use_exceptions=True)
             except ValueError:
                 error = 'The code you provided for FLEUR does not use the plugin fleur.fleur'
-                self.control_end_wc(error)
+                self.report(error)
                 return self.exit_codes.ERROR_INVALID_CODE_PROVIDED
 
         if 'scf' in inputs:
             self.ctx.scf_needed = True
             if 'remote' in inputs:
                 error = 'ERROR: you gave SCF input + remote for the BandDOS calculation'
-                self.control_end_wc(error)
+                self.report(error)
                 return self.exit_codes.ERROR_INVALID_INPUT_CONFIG
             if 'fleurinp' in inputs:
                 error = 'ERROR: you gave SCF input + fleurinp for the BandDOS calculation'
-                self.control_end_wc(error)
+                self.report(error)
                 return self.exit_codes.ERROR_INVALID_INPUT_CONFIG
         elif 'remote' not in inputs:
             error = 'ERROR: you gave neither SCF input nor remote'
-            self.control_end_wc(error)
+            self.report(error)
             return self.exit_codes.ERROR_INVALID_INPUT_CONFIG
         else:
             self.ctx.scf_needed = False
 
         if wf_dict['mode'] == 'dos' and wf_dict['kpath'] not in ('auto', 'skip'):
             error = 'ERROR: you specified the DOS mode but provided a non default kpath argument'
-            self.control_end_wc(error)
+            self.report(error)
             return self.exit_codes.ERROR_INVALID_INPUT_PARAM
 
-        if wf_dict['kpoints_number'] is not None and wf_dict['kpoints_number'] is not None:
+        if wf_dict['kpoints_number'] is not None and wf_dict['kpoints_distance'] is not None:
             error = 'ERROR: Only provide either the distance or number for the kpoints'
-            self.control_end_wc(error)
+            self.report(error)
             return self.exit_codes.ERROR_INVALID_INPUT_PARAM
 
     def change_fleurinp(self):
@@ -240,17 +251,16 @@ class FleurBandDosWorkChain(WorkChain):
                 self.control_end_wc(error)
                 return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
 
-        if listname is None:
-            if wf_dict.get('mode') == 'band':
-                listname = 'path-2'
+        if listname is None and wf_dict['mode'] == 'band':
+            listname = 'path-2'
 
         if nkpts is None and distance is None:
             nkpts = 500
 
         if 'kpoints' in self.inputs:
-            fleurmode.set_kpointsdata(self.inputs.kpoints, switch=True)
-
-        if kpath == 'auto':
+            kpoint_type = 'path' if wf_dict['mode'] == 'band' else 'mesh'
+            fleurmode.set_kpointsdata(self.inputs.kpoints, switch=True, kpoint_type=kpoint_type)
+        elif kpath == 'auto':
             if fleurin.inp_version >= '0.32' and listname is not None:
                 fleurmode.switch_kpointset(listname)
         elif isinstance(kpath, dict):
@@ -262,7 +272,7 @@ class FleurBandDosWorkChain(WorkChain):
                 raise ValueError('set_kpath is only supported for inputs up to Max4')
         elif kpath == 'seek':
             #Use aiida functionality
-            struc = fleurin.get_structuredata()
+            struc = fleurin.get_structuredata_ncf()
 
             if distance is not None:
                 output = get_explicit_kpoints_path(struc, reference_distance=distance)
@@ -284,10 +294,10 @@ class FleurBandDosWorkChain(WorkChain):
             fleurmode.set_kpointsdata(output['explicit_kpoints'], switch=True)
 
         elif kpath == 'skip':
-            return
+            pass
         else:
             #Use ase
-            struc = fleurin.get_structuredata()
+            struc = fleurin.get_structuredata_ncf()
 
             path = bandpath(kpath, cell=struc.cell, npoints=nkpts, density=distance)
 
@@ -313,11 +323,10 @@ class FleurBandDosWorkChain(WorkChain):
         emin = wf_dict['emin']
         emax = wf_dict['emax']
 
-        if fleurin.inp_version < '0.32':
-            if wf_dict.get('mode') == 'dos':
-                fleurmode.set_inpchanges({'ndir': -1})
+        if fleurin.inp_version < '0.32' and wf_dict['mode'] == 'dos':
+            fleurmode.set_inpchanges({'ndir': -1})
 
-        if wf_dict.get('mode') == 'dos':
+        if wf_dict['mode'] == 'dos':
             change_dict = {'dos': True, 'minEnergy': emin, 'maxEnergy': emax, 'sigma': sigma}
         else:
             change_dict = {'band': True, 'minEnergy': emin, 'maxEnergy': emax, 'sigma': sigma}
@@ -384,6 +393,10 @@ class FleurBandDosWorkChain(WorkChain):
             return status
 
         fleurin = self.ctx.fleurinp_banddos
+        if fleurin is None:
+            error = ('ERROR: Creating BandDOS Fleurinp failed for an unknown reason')
+            self.control_end_wc(error)
+            return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
 
         # Do not copy mixing_history* files from the parent
         settings = {'remove_from_remotecopy_list': ['mixing_history*']}
@@ -431,6 +444,10 @@ class FleurBandDosWorkChain(WorkChain):
             return status
 
         fleurin = self.ctx.fleurinp_banddos
+        if fleurin is None:
+            error = ('ERROR: Creating BandDOS Fleurinp failed for an unknown reason')
+            self.control_end_wc(error)
+            return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
 
         # Do not copy mixing_history* files from the parent
         settings = {'remove_from_remotecopy_list': ['mixing_history*']}
@@ -469,11 +486,11 @@ class FleurBandDosWorkChain(WorkChain):
         '''
         # TODO more here
         self.report('BandDOS workflow Done')
-        self.report(f'A bandstructure was calculated and is found under pk={self.ctx.banddos_calc.pk}, '
-                    f'calculation {self.ctx.banddos_calc}')
 
         from aiida_fleur.tools.common_fleur_wf import find_last_submitted_calcjob
         if self.ctx.banddos_calc:
+            self.report(f'A bandstructure/DOS was calculated and is found under pk={self.ctx.banddos_calc.pk}, '
+                        f'calculation {self.ctx.banddos_calc}')
             try:
                 last_calc_uuid = find_last_submitted_calcjob(self.ctx.banddos_calc)
             except NotExistent:
@@ -484,15 +501,20 @@ class FleurBandDosWorkChain(WorkChain):
         try:  # if something failed, we still might be able to retrieve something
             last_calc_out = self.ctx.banddos_calc.outputs.output_parameters
             retrieved = self.ctx.banddos_calc.outputs.retrieved
+            if 'fleurinpdata' in self.ctx.banddos_calc.inputs:
+                fleurinp = self.ctx.banddos_calc.inputs.fleurinpdata
+            else:
+                fleurinp = get_fleurinp_from_remote_data(self.ctx.banddos_calc.inputs.parent_folder)
             last_calc_out_dict = last_calc_out.get_dict()
         except (NotExistent, AttributeError):
             last_calc_out = None
             last_calc_out_dict = {}
             retrieved = None
+            fleurinp = None
 
         #check if band file exists: if not succesful = False
         #TODO be careful with general bands.X
-        bandfiles = ['bands.1', 'bands.2', 'banddos.hdf']
+        bandfiles = ['bands.1', 'bands.2', 'banddos.hdf', 'Local.1', 'Local.2']
 
         bandfile_res = []
         if retrieved:
@@ -502,18 +524,21 @@ class FleurBandDosWorkChain(WorkChain):
             if name in bandfile_res:
                 self.ctx.successful = True
         if not self.ctx.successful:
-            self.report('!NO bandstructure file was found, something went wrong!')
+            self.report('!NO bandstructure/DOS file was found, something went wrong!')
 
         # # get efermi from last calculation
         scf_results = None
         efermi_scf = 0
         bandgap_scf = 0
         if 'remote' in self.inputs:
-            for w in self.inputs.remote.get_incoming().all():
-                if isinstance(w.node, CalcJobNode):
-                    scf_results = load_node(w.node.pk).res
-                    efermi_scf = scf_results.fermi_energy
-                    bandgap_scf = scf_results.bandgap
+            scf_results = self.inputs.remote.creator.res
+        elif 'scf' in self.inputs:
+            if self.ctx.scf and self.ctx.scf.is_finished_ok:
+                scf_results = self.ctx.scf.outputs.last_calc.output_parameters.dict
+
+        if scf_results is not None:
+            efermi_scf = scf_results.fermi_energy
+            bandgap_scf = scf_results.bandgap
 
         efermi_band = last_calc_out_dict.get('fermi_energy', None)
         bandgap_band = last_calc_out_dict.get('bandgap', None)
@@ -532,7 +557,6 @@ class FleurBandDosWorkChain(WorkChain):
         outputnode_dict['Warnings'] = self.ctx.warnings
         outputnode_dict['successful'] = self.ctx.successful
         outputnode_dict['last_calc_uuid'] = last_calc_uuid
-        outputnode_dict['last_calc_pk'] = self.ctx.banddos_calc.pk
         outputnode_dict['mode'] = self.ctx.wf_dict.get('mode')
         outputnode_dict['fermi_energy_band'] = efermi_band
         outputnode_dict['bandgap_band'] = bandgap_band
@@ -548,6 +572,16 @@ class FleurBandDosWorkChain(WorkChain):
             outdict = create_band_result_node(outpara=outputnode_t,
                                               last_calc_out=last_calc_out,
                                               last_calc_retrieved=retrieved)
+
+            if self.ctx.wf_dict.get('mode') == 'band' and fleurinp is not None and retrieved is not None:
+                bands = create_aiida_bands_data(fleurinp=fleurinp, retrieved=retrieved)
+                if isinstance(bands, BandsData):
+                    outdict['output_banddos_wc_bands'] = bands
+            elif self.ctx.wf_dict.get('mode') == 'dos' and retrieved is not None:
+                dos = create_aiida_dos_data(retrieved=retrieved)
+                if isinstance(dos, XyData):
+                    outdict['output_banddos_wc_dos'] = dos
+
         else:
             outdict = create_band_result_node(outpara=outputnode_t)
 
@@ -588,3 +622,96 @@ def create_band_result_node(**kwargs):
     outdict['output_banddos_wc_para'] = outputnode
 
     return outdict
+
+
+@cf
+def create_aiida_bands_data(fleurinp, retrieved):
+    """
+    Creates :py:class:`aiida.orm.BandsData` object containing the kpoints and eigenvalues
+    from the `banddos.hdf` file of the calculation
+
+    :param fleurinp: :py:class:`~aiida_fleur.data.fleurinp.FleurinpData` for the calculation
+    :param retrieved: :py:class:`aiida.orm.FolderData` for the bandstructure calculation
+
+    :returns: :py:class:`aiida.orm.BandsData` for the bandstructure calculation
+
+    :raises: ExitCode 300, banddos.hdf file is missing
+    :raises: ExitCode 310, banddos.hdf reading failed
+    :raises: ExitCode 320, reading kpointsdata from Fleurinp failed
+    """
+    from masci_tools.io.parsers.hdf5 import HDF5Reader, HDF5TransformationError
+    from masci_tools.io.parsers.hdf5.recipes import FleurSimpleBands  #no projections only eigenvalues for now
+    from aiida.engine import ExitCode
+
+    try:
+        kpoints = fleurinp.get_kpointsdata_ncf(only_used=True)
+    except ValueError as exc:
+        return ExitCode(320, message=f'Retrieving kpoints data from fleurinp failed with: {exc}')
+
+    if 'banddos.hdf' in retrieved.list_object_names():
+        try:
+            with retrieved.open('banddos.hdf', 'rb') as f:
+                with HDF5Reader(f) as reader:
+                    data, attributes = reader.read(recipe=FleurSimpleBands)
+        except (HDF5TransformationError, ValueError) as exc:
+            return ExitCode(310, message=f'banddos.hdf reading failed with: {exc}')
+    else:
+        return ExitCode(300, message='banddos.hdf file not in the retrieved files')
+
+    bands = BandsData()
+    bands.set_kpointsdata(kpoints)
+
+    nkpts, nbands = attributes['nkpts'], attributes['nbands']
+    eigenvalues = data['eigenvalues_up'].reshape((nkpts, nbands))
+    if 'eigenvalues_down' in data:
+        eigenvalues_dn = data['eigenvalues_down'].reshape((nkpts, nbands))
+        eigenvalues = [eigenvalues, eigenvalues_dn]
+
+    bands.set_bands(eigenvalues, units='eV')
+
+    bands.label = 'output_banddos_wc_bands'
+    bands.description = ('Contains BandsData for the bandstructure calculation')
+
+    return bands
+
+
+@cf
+def create_aiida_dos_data(retrieved):
+    """
+    Creates :py:class:`aiida.orm.XyData` object containing the standard DOS components
+    from the `banddos.hdf` file of the calculation
+
+    :param retrieved: :py:class:`aiida.orm.FolderData` for the DOS calculation
+
+    :returns: :py:class:`aiida.orm.XyData` containing all standard DOS components
+
+    :raises: ExitCode 300, banddos.hdf file is missing
+    :raises: ExitCode 310, banddos.hdf reading failed
+    """
+    from masci_tools.io.parsers.hdf5 import HDF5Reader, HDF5TransformationError
+    from masci_tools.io.parsers.hdf5.recipes import FleurDOS  #only standard DOS for now
+    from aiida.engine import ExitCode
+
+    if 'banddos.hdf' in retrieved.list_object_names():
+        try:
+            with retrieved.open('banddos.hdf', 'rb') as f:
+                with HDF5Reader(f) as reader:
+                    data, attributes = reader.read(recipe=FleurDOS)
+        except (HDF5TransformationError, ValueError) as exc:
+            return ExitCode(310, message=f'banddos.hdf reading failed with: {exc}')
+    else:
+        return ExitCode(300, message='banddos.hdf file not in the retrieved files')
+
+    dos = XyData()
+    dos.set_x(data['energy_grid'], 'energy', x_units='eV')
+
+    names = [key for key in data if key != 'energy_grid']
+    arrays = [entry for key, entry in data.items() if key != 'energy_grid']
+    units = ['1/eV'] * len(names)
+    dos.set_y(arrays, names, y_units=units)
+
+    dos.label = 'output_banddos_wc_dos'
+    dos.description = (
+        'Contains XyData for the density of states calculation with total, interstitial, atom and orbital weights')
+
+    return dos
