@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 ###############################################################################
 # Copyright (c), Forschungszentrum Jülich GmbH, IAS-1/PGI-1, Germany.         #
 #                All rights reserved.                                         #
@@ -14,17 +13,16 @@
     Magnetic Anisotropy Energy via the force theorem.
 """
 
-from __future__ import absolute_import
 import copy
 
-import six
 #from six.moves import map
 from lxml import etree
+from ase.dft.kpoints import monkhorst_pack
 
 from aiida.engine import WorkChain, ToContext, if_
 from aiida.engine import calcfunction as cf
 from aiida.orm import Code, load_node
-from aiida.orm import RemoteData, Dict
+from aiida.orm import RemoteData, Dict, KpointsData
 from aiida.common import AttributeDict
 from aiida.common.exceptions import NotExistent
 
@@ -41,7 +39,7 @@ class FleurMaeWorkChain(WorkChain):
         This workflow calculates the Magnetic Anisotropy Energy of a structure.
     """
 
-    _workflowversion = '0.3.0'
+    _workflowversion = '0.3.1'
 
     _default_options = {
         'resources': {
@@ -61,11 +59,12 @@ class FleurMaeWorkChain(WorkChain):
         'sqas_theta': [0.0, 1.57079, 1.57079],
         'sqas_phi': [0.0, 0.0, 1.57079],
         'add_comp_para': {
-            'serial': False,
             'only_even_MPI': False,
             'max_queue_nodes': 20,
             'max_queue_wallclock_sec': 86400
         },
+        'kmesh_force_theorem': None,
+        'use_symmetries_reference': False,
         'soc_off': [],
         'inpxml_changes': [],
     }
@@ -113,8 +112,7 @@ class FleurMaeWorkChain(WorkChain):
         """
         Retrieve and initialize paramters of the WorkChain
         """
-        self.report('INFO: started Magnetic Anisotropy Energy calculation workflow version {}\n'
-                    ''.format(self._workflowversion))
+        self.report(f'INFO: started Magnetic Anisotropy Energy calculation workflow version {self._workflowversion}\n')
 
         self.ctx.info = []
         self.ctx.warnings = []
@@ -136,12 +134,12 @@ class FleurMaeWorkChain(WorkChain):
             if key not in wf_default.keys():
                 extra_keys.append(key)
         if extra_keys:
-            error = 'ERROR: input wf_parameters for MAE contains extra keys: {}'.format(extra_keys)
+            error = f'ERROR: input wf_parameters for MAE contains extra keys: {extra_keys}'
             self.report(error)
             return self.exit_codes.ERROR_INVALID_INPUT_PARAM
 
         # extend wf parameters given by user using defaults
-        for key, val in six.iteritems(wf_default):
+        for key, val in wf_default.items():
             wf_dict[key] = wf_dict.get(key, val)
         self.ctx.wf_dict = wf_dict
 
@@ -171,7 +169,7 @@ class FleurMaeWorkChain(WorkChain):
             options = defaultoptions
 
         # extend options given by user using defaults
-        for key, val in six.iteritems(defaultoptions):
+        for key, val in defaultoptions.items():
             options[key] = options.get(key, val)
         self.ctx.options = options
 
@@ -239,7 +237,17 @@ class FleurMaeWorkChain(WorkChain):
             scf_wf_dict['inpxml_changes'].append(('set_inpchanges', {'change_dict': {'l_soc': False}}))
         else:  # set soc parameters explicitly
             changes_dict = {'theta': soc[0], 'phi': soc[1], 'l_soc': True}
-            scf_wf_dict['inpxml_changes'].append(('set_inpchanges', {'change_dict': changes_dict}))
+            scf_wf_dict['inpxml_changes'].append(('set_inpchanges', {
+                'change_dict': changes_dict,
+                'path_spec': {
+                    'phi': {
+                        'contains': 'soc'
+                    },
+                    'theta': {
+                        'contains': 'soc'
+                    }
+                }
+            }))
 
         input_scf.wf_parameters = Dict(dict=scf_wf_dict)
 
@@ -248,7 +256,9 @@ class FleurMaeWorkChain(WorkChain):
                 calc_parameters = input_scf.calc_parameters.get_dict()
             else:
                 calc_parameters = {}
-            calc_parameters['soc'] = {'theta': soc[0], 'phi': soc[1]}
+            if not self.ctx.wf_dict.get('use_symmetries_reference'):
+                # break symmetries, SOC will be removed if not set
+                calc_parameters['soc'] = {'theta': soc[0], 'phi': soc[1]}
             input_scf.calc_parameters = Dict(dict=calc_parameters)
 
         return input_scf
@@ -277,52 +287,43 @@ class FleurMaeWorkChain(WorkChain):
         fchanges = self.ctx.wf_dict.get('inpxml_changes', [])
 
         # add forceTheorem tag into inp.xml
-        fchanges.extend([
-            ('create_tag', {
-                'xpath': '/fleurInput',
-                'newelement': 'forceTheorem'
-            }),
-            ('create_tag', {
-                'xpath': '/fleurInput/forceTheorem',
-                'newelement': 'MAE'
-            }),
-            ('xml_set_attribv_occ', {
-                'xpathn': '/fleurInput/forceTheorem/MAE',
-                'attributename': 'theta',
-                'attribv': ' '.join(six.moves.map(str, self.ctx.wf_dict.get('sqas_theta')))
-            }),
-            ('xml_set_attribv_occ', {
-                'xpathn': '/fleurInput/forceTheorem/MAE',
-                'attributename': 'phi',
-                'attribv': ' '.join(six.moves.map(str, self.ctx.wf_dict.get('sqas_phi')))
-            }),
-            ('set_inpchanges', {
-                'change_dict': {
-                    'itmax': 1,
-                    'l_soc': True
-                }
-            }),
-        ])
+        fchanges.append(('set_complex_tag', {
+            'tag_name': 'MAE',
+            'create': True,
+            'changes': {
+                'theta': self.ctx.wf_dict['sqas_theta'],
+                'phi': self.ctx.wf_dict['sqas_phi']
+            }
+        }))
+        fchanges.append(('set_inpchanges', {'change_dict': {'itmax': 1, 'l_soc': True}}),)
+
+        if self.ctx.wf_dict['kmesh_force_theorem'] is not None:
+            # set k-mesh for the full BZ
+            kmesh = KpointsData()
+            kmesh.set_kpoints(monkhorst_pack(self.ctx.wf_dict['kmesh_force_theorem']))
+            kmesh.store()
+            fchanges.append(('set_kpointsdata', {
+                'kpointsdata_uuid': kmesh.uuid,
+                'switch': True,
+                'kpoint_type': 'mesh'
+            }))
+
+        # if self.ctx.wf_dict['use_symmetries_reference']:
+        #     # remove symmetries from the inp.xml
+        #     fchanges.append(('delete_tag', {
+        #         'tag_name': 'symOp',
+        #         'occurrences': range(1, len(fleurin.inp_dict['cell']['symmetryOperations']))
+        #     }))
 
         if fchanges:  # change inp.xml file
             fleurmode = FleurinpModifier(fleurin)
-            avail_ac_dict = fleurmode.get_avail_actions()
-
-            # apply further user dependend changes
-            for change in fchanges:
-                function = change[0]
-                para = change[1]
-                method = avail_ac_dict.get(function, None)
-                if not method:
-                    error = ("ERROR: Input 'inpxml_changes', function {} "
-                             'is not known to fleurinpmodifier class, '
-                             'please check/test your input. I abort...'
-                             ''.format(function))
-                    self.control_end_wc(error)
-                    return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
-
-                else:  # apply change
-                    method(**para)
+            try:
+                fleurmode.add_task_list(fchanges)
+            except (ValueError, TypeError) as exc:
+                error = ('ERROR: Changing the inp.xml file failed. Tried to apply inpxml_changes'
+                         f', which failed with {exc}. I abort, good luck next time!')
+                self.control_end_wc(error)
+                return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
 
             # validate?
             try:
@@ -331,14 +332,19 @@ class FleurMaeWorkChain(WorkChain):
                 error = ('ERROR: input, user wanted inp.xml changes did not validate')
                 self.report(error)
                 return self.exit_codes.ERROR_INVALID_INPUT_FILE
+            except ValueError as exc:
+                error = ('ERROR: input, user wanted inp.xml changes could not be applied.'
+                         f'The following error was raised {exc}')
+                self.control_end_wc(error)
+                return self.exit_codes.ERROR_CHANGING_FLEURINPUT_FAILED
 
             # apply
             out = fleurmode.freeze()
             self.ctx.fleurinp = out
-            return
         else:  # otherwise do not change the inp.xml
             self.ctx.fleurinp = fleurin
-            return
+
+        return
 
     def force_after_scf(self):
         """
@@ -457,8 +463,7 @@ class FleurMaeWorkChain(WorkChain):
         try:
             calculation = self.ctx.f_t
             if not calculation.is_finished_ok:
-                message = ('ERROR: Force theorem Fleur calculation failed somehow it has '
-                           'exit status {}'.format(calculation.exit_status))
+                message = f'ERROR: Force theorem Fleur calculation failed somehow it has exit status {calculation.exit_status}'
                 self.control_end_wc(message)
                 return self.exit_codes.ERROR_FORCE_THEOREM_FAILED
         except AttributeError:
@@ -483,7 +488,7 @@ class FleurMaeWorkChain(WorkChain):
                 t_energydict = [(x - minenergy) for x in t_energydict]
 
         except AttributeError as e_message:
-            message = ('Did not manage to read evSum or energy units after FT calculation. {}'.format(e_message))
+            message = f'Did not manage to read evSum or energy units after FT calculation. {e_message}'
             self.control_end_wc(message)
             return self.exit_codes.ERROR_FORCE_THEOREM_FAILED
 
@@ -542,7 +547,7 @@ def save_mae_output_node(**kwargs):
     So far it is just also parsed in as argument, because so far we are to lazy
     to put most of the code overworked from return_results in here.
     """
-    for key, val in six.iteritems(kwargs):
+    for key, val in kwargs.items():
         if key == 'out':  # should be always there
             outpara = val
     outdict = {}
