@@ -17,7 +17,7 @@ options = Dict(dict={'resources': {'num_machines': 2, 'num_mpiprocs_per_machine'
 
 wf_para_corehole = Dict(dict={u'atoms': [u'Be'], #[u'all'],
   u'supercell_size': [2, 2, 2], u'corelevel': ['1s'], #[u'all'],
-  u'hole_charge': 1.0, u'magnetic': True, u'method': u'valence', u'serial': False})
+  u'hole_charge': 1.0, u'magnetic': True, u'method': u'valence'})
 
 # launch workflow
 dos = submit(fleur_corehole_wc, wf_parameters=wf_para_corehole, structure=struc,
